/******************** (C) COPYRIGHT 2009 jasys GmbH & Co. KG *******************
 * Name    : Init.c
 * Autor   : joerg
 * Erstellt: 16.03.2009
 * ******************************************************************************
 * Version | Datum      | Beschreibung
 ----------+------------+------------------------------------------------------------
 * 1.00.00 | 16.03.2009 | Datei erstellt
 *
 *******************************************************************************/

#include "tft.h"
#include "stm32f407xx.h"
#include "stm32f4xx_hal.h"
#include "fsmc.h"
#include "adc.h"
#include "EMG_Emg.h"
#include "EMG_IO.h"

/* Private typedef -----------------------------------------------------------*/

/* Private define ------------------------------------------------------------*/
#define SRAM_BANK_ADDR ((uint32_t)0x60000000) // SRAM BANK 1 ADDRESS
/* Private macro -------------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/
uint16_t u16Bild1[BILD_PART1_GROESSE];
uint16_t u8TFTUpdateNecessary = 0;

// recommend to write a recalibration function
int16_t i16Touch_X_CALMIN = 4000;
int16_t i16Touch_X_CALMAX = 100;
int16_t i16Touch_Y_CALMIN = 100;
int16_t i16Touch_Y_CALMAX = 4000;

sTouch_t Touch;

/* Private function prototypes -----------------------------------------------*/

/* Private functions ---------------------------------------------------------*/

/*
 * ST7789S delay
 */
void TFT_delay ( uint16_t u16Waittime )
{
    u16TFT_Delay = u16Waittime;
    while ( u16TFT_Delay > 0 )
    {
    }
}

/*
 * write command with waittime to display
 */
void TFT_24S_Write_Command_Wait ( uint16_t u16command )
{
    HAL_GPIO_WritePin(GPIOD, GPIO_PIN_3, GPIO_PIN_RESET);
    *(uint16_t *) (SRAM_BANK_ADDR ) = u16command;

    TFT_delay(20);

    HAL_GPIO_WritePin(GPIOD, GPIO_PIN_3, GPIO_PIN_SET);
}

/*
 * write command to display
 */
void TFT_24S_Write_Command_NoWait ( uint16_t u16command )
{
    HAL_GPIO_WritePin(GPIOD, GPIO_PIN_3, GPIO_PIN_RESET);
    *(uint16_t *) (SRAM_BANK_ADDR ) = u16command;

    TFT_delay(10);

    HAL_GPIO_WritePin(GPIOD, GPIO_PIN_3, GPIO_PIN_SET);
}

/*
 * write data to display
 */
void TFT_24S_Write_Data ( uint16_t u16data )
{
    *(uint16_t *) (SRAM_BANK_ADDR ) = u16data;
}


/*
 * initalize the chip from the TFT
 */
void TFT_24_7789_Init(void)
{

	TFT_delay(100);
	TFT_24S_Write_Command_Wait(0x0011); //exit SLEEP mode
	TFT_delay(100);

	TFT_24S_Write_Command_Wait(0x0036);
	TFT_24S_Write_Data(0x0080); //MADCTL: memory data access control
	TFT_24S_Write_Command_Wait(0x003A);
	TFT_24S_Write_Data(0x0065); //COLMOD: Interface Pixel format *** I use 262K-colors in 18bit/pixel format when using 8-bit interface to allow 3-bytes per pixel
	//TFT_24_7789_Write_Command(0x003A);TFT_24_7789_Write_Data(0x0055);//COLMOD: Interface Pixel format  *** I use 65K-colors in 16bit/pixel (5-6-5) format when using 16-bit interface to allow 1-byte per pixel
	TFT_24S_Write_Command_Wait(0x00B2);
	TFT_24S_Write_Data(0x000C);
	TFT_24S_Write_Data(0x0C);
	TFT_24S_Write_Data(0x00);
	TFT_24S_Write_Data(0x33);
	TFT_24S_Write_Data(0x33); //PORCTRK: Porch setting
	TFT_24S_Write_Command_Wait(0x00B7);
	TFT_24S_Write_Data(0x0035); //GCTRL: Gate Control
	TFT_24S_Write_Command_Wait(0x00BB);
	TFT_24S_Write_Data(0x002B); //VCOMS: VCOM setting
	TFT_24S_Write_Command_Wait(0x00C0);
	TFT_24S_Write_Data(0x002C); //LCMCTRL: LCM Control
	TFT_24S_Write_Command_Wait(0x00C2);
	TFT_24S_Write_Data(0x0001);
	TFT_24S_Write_Data(0xFF); //VDVVRHEN: VDV and VRH Command Enable
	TFT_24S_Write_Command_Wait(0x00C3);
	TFT_24S_Write_Data(0x0011); //VRHS: VRH Set
	TFT_24S_Write_Command_Wait(0x00C4);
	TFT_24S_Write_Data(0x0020); //VDVS: VDV Set
	TFT_24S_Write_Command_Wait(0x00C6);
	TFT_24S_Write_Data(0x000F); //FRCTRL2: Frame Rate control in normal mode
	TFT_24S_Write_Command_Wait(0x00D0);
	TFT_24S_Write_Data(0x00A4);
	TFT_24S_Write_Data(0xA1); //PWCTRL1: Power Control 1

	TFT_24S_Write_Command_Wait(0x00E0);
	TFT_24S_Write_Data(0x00D0);
	TFT_24S_Write_Data(0x0000);
	TFT_24S_Write_Data(0x0005);
	TFT_24S_Write_Data(0x000E);
	TFT_24S_Write_Data(0x0015);
	TFT_24S_Write_Data(0x000D);
	TFT_24S_Write_Data(0x0037);
	TFT_24S_Write_Data(0x0043);
	TFT_24S_Write_Data(0x0047);
	TFT_24S_Write_Data(0x0009);
	TFT_24S_Write_Data(0x0015);
	TFT_24S_Write_Data(0x0012);
	TFT_24S_Write_Data(0x0016);
	TFT_24S_Write_Data(0x0019);//PVGAMCTRL: Positive Voltage Gamma control

	TFT_24S_Write_Command_Wait(0x00E1);TFT_24S_Write_Data(0x00D0);
	TFT_24S_Write_Data(0x0000);
	TFT_24S_Write_Data(0x0005);
	TFT_24S_Write_Data(0x000D);
	TFT_24S_Write_Data(0x000C);
	TFT_24S_Write_Data(0x0006);
	TFT_24S_Write_Data(0x002D);
	TFT_24S_Write_Data(0x0044);
	TFT_24S_Write_Data(0x0040);
	TFT_24S_Write_Data(0x000E);
	TFT_24S_Write_Data(0x001C);
	TFT_24S_Write_Data(0x0018);
	TFT_24S_Write_Data(0x0016);
	TFT_24S_Write_Data(0x0019);//NVGAMCTRL: Negative Voltage Gamma control

	TFT_24S_Write_Command_Wait(0x002A);
	TFT_24S_Write_Data(0x0000);
	TFT_24S_Write_Data(0x0000);
	TFT_24S_Write_Data(0x0000);
	TFT_24S_Write_Data(0x00EF); //X address set
	TFT_24S_Write_Command_Wait(0x002B);
	TFT_24S_Write_Data(0x0000);
	TFT_24S_Write_Data(0x0000);
	TFT_24S_Write_Data(0x0001);
	TFT_24S_Write_Data(0x003F); //Y address set
	TFT_delay(10);
	
	TFT_24S_Write_Command_Wait(0x0029); //display ON
}


/*
 * update screen from cpu ram to display
 */
void vDisplayupdate ( void )
{
    uint32_t u32i = 0;

    if ( !u8TFTUpdateNecessary )
        return;
    u8TFTUpdateNecessary = 0;

    TFT_24S_Write_Command_NoWait(0x002C); // Memory write

    for (u32i = 0; u32i < BILD_PART0_GROESSE; u32i++)
        TFT_24S_Write_Data(u16Bild0[u32i]);
    for (u32i = 0; u32i < BILD_PART1_GROESSE; u32i++)
        TFT_24S_Write_Data(u16Bild1[u32i]);
}

/*
 * clear screen with one color
 */
void vCls ( uint16_t u16Color )
{
    uint32_t u32i = 0;

    for (u32i = 0; u32i < BILD_PART0_GROESSE; u32i++)
        u16Bild0[u32i] = u16Color;
    for (u32i = 0; u32i < BILD_PART1_GROESSE; u32i++)
        u16Bild1[u32i] = u16Color;
}

/*******************************************************************************
 * Function Name  : vTouchGetPos
 * Description    : Check the touch for pressed event, if yes then start the ADC
 *                  read in from coordinates of presses display.
 * Input          : None.
 * Output         : None.
 * Return         : None.
 *******************************************************************************/
void vTouchGetPos ( void )
{
    static GPIO_InitTypeDef GPIO_InitStructure;
    ADC_ChannelConfTypeDef sConfig;
    static uint16_t u16TouchX[2] = {0, 0};
    static uint16_t u16TouchY[2] = {0, 0};
    static uint8_t u8Factor = 4;
    static uint8_t u8Count = 0;
    static uint32_t u32I;
    static uint8_t u8Ersterdurchlauf = 0;
    static uint32_t u32Schleifenende = 100;

    if ( u16Touch_Delay == 0 )
    {
        if ( (HAL_GPIO_ReadPin(GPIOC, GPIO_PIN_0 ) == 0) && (HAL_GPIO_ReadPin(GPIOC, GPIO_PIN_2 ) == 0) )
        {
            Touch.u8appraise = 1;
            u8Count++;
        }
        else
        {
            u8Ersterdurchlauf = 0;
            u8Count = 0;
            Touch.u8pressed = 0;
        }

        if ( Touch.u8appraise == 1 )
        {
            Touch.u8appraise = 0;

            GPIO_InitStructure.Pin = GPIO_PIN_0;
            GPIO_InitStructure.Mode = GPIO_MODE_ANALOG;
            GPIO_InitStructure.Speed = GPIO_SPEED_FREQ_HIGH;
            GPIO_InitStructure.Pull = GPIO_NOPULL;
            HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);

            GPIO_InitStructure.Pin = GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3;
            GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
            GPIO_InitStructure.Speed = GPIO_SPEED_FREQ_HIGH;
            GPIO_InitStructure.Pull = GPIO_NOPULL;
            HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);

            HAL_GPIO_WritePin(GPIOC, GPIO_PIN_1, GPIO_PIN_SET);
            HAL_GPIO_WritePin(GPIOC, GPIO_PIN_2, GPIO_PIN_RESET);
            HAL_GPIO_WritePin(GPIOC, GPIO_PIN_3, GPIO_PIN_RESET);
            GPIO_InitStructure.Pin = GPIO_PIN_2;
            GPIO_InitStructure.Mode = GPIO_MODE_ANALOG;
            GPIO_InitStructure.Speed = GPIO_SPEED_FREQ_HIGH;
            GPIO_InitStructure.Pull = GPIO_NOPULL;
            HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);

            sConfig.Channel = ADC_CHANNEL_10;
            sConfig.Rank = 1;
            sConfig.SamplingTime = ADC_SAMPLETIME_3CYCLES;
            HAL_ADC_ConfigChannel(&hadc1, &sConfig);

            for (u32I = 0; u32I < u32Schleifenende; u32I++)
            {
            }

            HAL_ADC_Start(&hadc1);
            HAL_ADC_PollForConversion(&hadc1, 100);

            u16TouchY[1] = HAL_ADC_GetValue(&hadc1); //get value

            if ( u8Ersterdurchlauf == 0 )
                u16TouchY[0] = u16TouchY[1] * u8Factor;
            else
            {
                u16TouchY[0] = u16TouchY[0] + u16TouchY[1] - u16TouchY[0] / u8Factor;
                Touch.i16Y_raw = u16TouchY[0] / u8Factor;
            }

            GPIO_InitStructure.Pin = GPIO_PIN_1;
            GPIO_InitStructure.Mode = GPIO_MODE_ANALOG;
            GPIO_InitStructure.Speed = GPIO_SPEED_FREQ_HIGH;
            GPIO_InitStructure.Pull = GPIO_NOPULL;
            HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);

            GPIO_InitStructure.Pin = GPIO_PIN_0 | GPIO_PIN_2 | GPIO_PIN_3;
            GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
            GPIO_InitStructure.Speed = GPIO_SPEED_FREQ_HIGH;
            GPIO_InitStructure.Pull = GPIO_NOPULL;
            HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);

            HAL_GPIO_WritePin(GPIOC, GPIO_PIN_0, GPIO_PIN_RESET);
            HAL_GPIO_WritePin(GPIOC, GPIO_PIN_2, GPIO_PIN_SET);
            HAL_GPIO_WritePin(GPIOC, GPIO_PIN_3, GPIO_PIN_RESET);

            GPIO_InitStructure.Pin = GPIO_PIN_3;
            GPIO_InitStructure.Mode = GPIO_MODE_ANALOG;
            GPIO_InitStructure.Speed = GPIO_SPEED_FREQ_HIGH;
            GPIO_InitStructure.Pull = GPIO_NOPULL;
            HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);

            sConfig.Channel = ADC_CHANNEL_11;
            sConfig.Rank = 1;
            sConfig.SamplingTime = ADC_SAMPLETIME_3CYCLES;

            HAL_ADC_ConfigChannel(&hadc1, &sConfig);

            for (u32I = 0; u32I < u32Schleifenende; u32I++)
            {
            }

            HAL_ADC_Start(&hadc1);
            HAL_ADC_PollForConversion(&hadc1, 100);

            u16TouchX[1] = HAL_ADC_GetValue(&hadc1); //get value

            if ( u8Ersterdurchlauf == 0 )
            {
                u8Ersterdurchlauf = 1;
                u16TouchX[0] = u16TouchX[1] * u8Factor;
            }
            else
            {
                u16TouchX[0] = u16TouchX[0] + u16TouchX[1] - u16TouchX[0] / u8Factor;
                Touch.i16X_raw = u16TouchX[0] / u8Factor;
            }

            if ( u8Count >= u8Factor )
            {
                u8Count = u8Factor;
                Touch.u8pressed = 1;
                Touch.i16X = (((u16TouchX[0] / u8Factor) - i16Touch_X_CALMIN) * 320) / (i16Touch_X_CALMAX - i16Touch_X_CALMIN);
                Touch.i16Y = (((u16TouchY[0] / u8Factor) - i16Touch_Y_CALMIN) * 240) / (i16Touch_Y_CALMAX - i16Touch_Y_CALMIN);

                if ( (Touch.i16X < 0) || (Touch.i16Y < 0) || (Touch.i16X > DIRECTION_X) || (Touch.i16Y > DIRECTION_Y) )
                {
                    Touch.i16Y = Touch.i16Y_last;
                    Touch.i16X = Touch.i16X_last;
                }
                Touch.i16X_last = Touch.i16X;
                Touch.i16Y_last = Touch.i16Y;
            }

            GPIO_InitStructure.Pin = GPIO_PIN_0 | GPIO_PIN_2;
            GPIO_InitStructure.Mode = GPIO_MODE_INPUT;
            GPIO_InitStructure.Speed = GPIO_SPEED_FREQ_HIGH;
            GPIO_InitStructure.Pull = GPIO_PULLUP;
            HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);

            GPIO_InitStructure.Pin = GPIO_PIN_1 | GPIO_PIN_3;
            GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
            GPIO_InitStructure.Speed = GPIO_SPEED_FREQ_HIGH;
            GPIO_InitStructure.Pull = GPIO_NOPULL;
            HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);

            HAL_GPIO_WritePin(GPIOC, GPIO_PIN_1, GPIO_PIN_RESET);
            HAL_GPIO_WritePin(GPIOC, GPIO_PIN_3, GPIO_PIN_RESET);
        }
        u16Touch_Delay = 1; //start 1ms delay timer
    }
}


