/*
 * EMG_IO.c
 *
 *  Created on: 24.01.2014
 *      Author: joerg
 */

#include "EMG_IO.h"
#include "EMG_Data.h"
#include "EMG_emg.h"

#include "main.h"

#if defined ( __GNUC__ ) /*------------------ GNU Compiler ---------------------*/
#define __packed
#include "string.h"
#endif

//typedef __packed union
typedef union
{
    uint32_t u32;
    uint16_t u16[2];
    uint8_t u8[4];
} uColor_t;

//uint8_t *g_au8Data;
//uint8_t *g_au8Font;
//uint8_t *g_au8String;
uint8_t *g_au8Image;

/*
 * choose display type
 * 0 = 1  bit
 * 1 = 8  bit gray color
 * 2 = 8  bit bit color
 * 3 = 12 bit color
 * 4 = 15 bit color
 * 5 = 16 bit color
 * 6 = 31 bit color
 * 7 = 32 bit color
 */
uint8_t g_u8DisplayType = 5;

//uint32_t *g_au32Canvas;
uint32_t g_u32Sizes[4];

//--------------------------- interface functions -----------------------------
// External functions, called by libraray

// Get Data from Data - Memory
uint8_t EMG_GETD_U8 ( uint32_t u32Adr )
{
	return *(uint8_t*) (EMG_START_DATA+u32Adr);
}
uint16_t EMG_GETD_U16 ( uint32_t u32Adr )
{
	return *(uint16_t*) (EMG_START_DATA+u32Adr);
}
uint32_t EMG_GETD_U32 ( uint32_t u32Adr )
{
	return *(uint32_t*) (EMG_START_DATA+u32Adr);
}

// Get Data from Font - Memory
uint8_t EMG_GETF_U8 ( uint32_t u32Adr )
{
	return *(uint8_t*) (EMG_START_FONT+u32Adr);
}
uint16_t EMG_GETF_U16 ( uint32_t u32Adr )
{
	return *(uint16_t*) (EMG_START_FONT+u32Adr);
}
uint32_t EMG_GETF_U32 ( uint32_t u32Adr )
{
	return *(uint32_t*) (EMG_START_FONT+u32Adr);
}

// Get Data from String - Memory
uint8_t EMG_GETS_U8 ( uint32_t u32Adr )
{
	return *(uint8_t*) (EMG_START_STRING+u32Adr);
}
uint16_t EMG_GETS_U16 ( uint32_t u32Adr )
{
	return *(uint16_t*) (EMG_START_STRING+u32Adr);
}
uint32_t EMG_GETS_U32 ( uint32_t u32Adr )
{
	return *(uint32_t*) (EMG_START_STRING+u32Adr);
}

// Get Data from Image - Memory
uint8_t EMG_GETI_U8 ( uint32_t u32Adr )
{
	return *(uint8_t*) (EMG_START_PICTURE+u32Adr);
}
uint16_t EMG_GETI_U16 ( uint32_t u32Adr )
{
	return *(uint16_t*) (EMG_START_PICTURE+u32Adr);
}
uint32_t EMG_GETI_U32 ( uint32_t u32Adr )
{
	return *(uint32_t*) (EMG_START_PICTURE+u32Adr);
}

//-----------------------------------------------------------------------------
/*
 Monochrome (b/w)
 Gray scale (8 Bit)
 Color (8 Bit, 3R/3G/2B)
 Color (15 Bit, 5R/5G/5B
 Color (16 Bit, 5R/6G/5B)
 Color (24 Bit, RGB)
 Color (32 Bit, ARGB)
 DisplayTypes_t = (dt1Bit, dt8BitG, dt8BitC, dt15Bit, dt16Bit, dt24Bit, dt32Bit);
 */
uint32_t EmgGetPixel ( uint16_t u16X, uint16_t u16Y )
{
    uint32_t u32SpeicherPlatz;
    uint32_t u32Color;

    if ( (u16X >= DIRECTION_X) || (u16Y >= DIRECTION_Y) )
        return 0;

    u32SpeicherPlatz = (319 - u16X) * 240 + (239 - u16Y);

    if ( u32SpeicherPlatz < BILD_PART0_GROESSE)
    {
        u32Color = u16Bild0[u32SpeicherPlatz];
    }
    else
    {
        u32SpeicherPlatz -= BILD_PART0_GROESSE;
        u32Color = u16Bild1[u32SpeicherPlatz];
    }

    return u32Color;
}

void EmgPutPixel ( uint16_t u16X, uint16_t u16Y, uint32_t u32Color )
{
    uint32_t u32SpeicherPlatz;

    if ( (u16X >= DIRECTION_X) || (u16Y >= DIRECTION_Y) ) //out of Range
        return;
    else
        u32SpeicherPlatz = (319 - u16X) * 240 + (239 - u16Y);

    if ( u32SpeicherPlatz < BILD_PART0_GROESSE)
    {
        u16Bild0[u32SpeicherPlatz] = u32Color;
    }
    else
    {
        u32SpeicherPlatz -= BILD_PART0_GROESSE;
        u16Bild1[u32SpeicherPlatz] = u32Color;
    }
    u8TFTUpdateNecessary = 1;
}

/*
 * clear screen with Color
 */
void EmgCls ( uint32_t u32Color )
{
    uint16_t xp, yp;

    for (yp = 0; yp < DIRECTION_Y; yp++)
    {
        for (xp = 0; xp < DIRECTION_X; xp++)
        {
            EmgPutPixel(xp, yp, u32Color);
        }
    }
}

/*
 * Touchhandler
 * check for pressed, hold or released touch screen
 */
void EmgTouchHandler ( uint16_t u16X, uint16_t u16Y, uint8_t u8NewState )
{
    static uint16_t u16OldState = 0;

    if ( (u16OldState == 0) && (u8NewState == 1) ) // pressed
    {
        u16OldState = u8NewState;
        //EMG call function
        EmgCheckUpdatesForActivePage(u16X, u16Y, 1); //not need to do something
    }
    else if ( (u16OldState == 1) && (u8NewState == 0) ) // released
    {
        u16OldState = u8NewState;
        //EMG call function
        EmgCheckUpdatesForActivePage(u16X, u16Y, 0); //look for action to do
    }
    else if ( (u16OldState == 1) && (u8NewState == 1) ) // hold
    {
        u16OldState = u8NewState;
    }
}

